-- Export list of players into a copy&paste'able box
-- 50

local core = CalendarExtras

local module = core:NewModule("Export")

 --[[ ------- Locals ------- ]]--

local exportFrame, exportFrameTitle, exportFrameCloseButton -- The backgtound frame
local exportScrollContainer, exportScrollFrame, focusButton, exportEditBox -- The scrolling editbox
local exportFormatEditBox -- The export format editbox

local Export

local defaultFormatStr = "NAME%tCLASS%tSTATUS"

local extraFormatTokens = {} -- Extra replacement tokens

local inviteStatusTable = {
	[CALENDAR_INVITESTATUS_CONFIRMED] = CALENDAR_STATUS_CONFIRMED,
	[CALENDAR_INVITESTATUS_ACCEPTED] = CALENDAR_STATUS_ACCEPTED,
	[CALENDAR_INVITESTATUS_DECLINED] = CALENDAR_STATUS_DECLINED,
	[CALENDAR_INVITESTATUS_OUT] = CALENDAR_STATUS_OUT,
	[CALENDAR_INVITESTATUS_STANDBY] = CALENDAR_STATUS_STANDBY,
	[CALENDAR_INVITESTATUS_INVITED] = CALENDAR_STATUS_INVITED,
	[CALENDAR_INVITESTATUS_SIGNEDUP] = CALENDAR_STATUS_SIGNEDUP,
	[CALENDAR_INVITESTATUS_NOT_SIGNEDUP] = CALENDAR_STATUS_NOT_SIGNEDUP,
	[CALENDAR_INVITESTATUS_TENTATIVE] = CALENDAR_STATUS_TENTATIVE
}

local tabList = {
	"CalendarExtrasExportScrollFrameEditBox",
	"CalendarExtrasExportFormatEditBox"
}
local function TabHandler(self)
	EditBox_HandleTabbing(self, tabList)
end

 --[[ -- Main frame setup -- ]]--

local function SetupFrames()
	exportFrame = CreateFrame("Frame", "CalendarExtrasExportFrame", CalendarFrame)
	exportFrame:SetWidth(320)
	exportFrame:SetHeight(580)
	exportFrame:SetPoint("TOPLEFT", CalendarFrame, "TOPRIGHT", 0, -24)
	exportFrame:SetBackdrop({
		bgFile = "",
		edgeFile = [[Interface\DialogFrame\UI-DialogBox-Border]],
		insets = {left = 11, right = 12, top = 12, bottom = 11},
		edgeSize = 32, tileSize = 32,
		tile = true,
	})
	exportFrame:Hide()
	local level1, level2 = CalendarViewEventFrame:GetFrameLevel(), CalendarCreateEventFrame:GetFrameLevel()
	exportFrame:SetFrameLevel(level1 > level2 and level1 or level2)
	exportFrame:SetFrameStrata("DIALOG")

	local texture = exportFrame:CreateTexture("CalendarExtrasExportFrameBackground", "BACKGROUND")
	texture:SetPoint("TOPLEFT", 10, -10)
	texture:SetPoint("BOTTOMRIGHT", -10, 10)
	texture:SetTexture(0, 0, 0, 0.8)

	exportFrameTitle = CreateFrame("Frame", "CalendarExtrasExportFrameTitle", exportFrame, "CalendarTitleFrameTemplate")
	local textFrame = _G["CalendarExtrasExportFrameTitleText"]
	local middleFrame = _G["CalendarExtrasExportFrameTitleBackgroundMiddle"]
	textFrame:SetText("Export")
	middleFrame:SetWidth(100)
	textFrame:SetWidth(100)

	exportFrameCloseButton = CreateFrame("Button", "CalendarExtrasExportFrameCloseButton", exportFrame, "CalendarCloseButtonTemplate")

	-- Editbox

	exportScrollContainer = CreateFrame("Frame", "CalendarExtrasExportScrollFrameContainer", exportFrame)
	exportScrollContainer:SetWidth(287)
	exportScrollContainer:SetPoint("TOPLEFT", 16, -32)
	exportScrollContainer:SetPoint("BOTTOMLEFT", 16, 32)
	exportScrollContainer:SetBackdrop({
		bgFile = [[Interface\Tooltips\UI-Tooltip-Background]],
		edgeFile = [[Interface\Tooltips\UI-Tooltip-Border]],
		insets = {left = 1, right = 1, top = 1, bottom = 1},
		edgeSize = 8, tileSize = 8,
		tile = true,
	})
	exportScrollContainer:SetBackdropBorderColor(TOOLTIP_DEFAULT_COLOR.r, TOOLTIP_DEFAULT_COLOR.g, TOOLTIP_DEFAULT_COLOR.b)
	exportScrollContainer:SetBackdropColor(0.0, 0.0, 0.0, 0.9)

	exportScrollFrame = CreateFrame("ScrollFrame", "CalendarExtrasExportScrollFrame", exportScrollContainer, "CalendarEventDescriptionScrollFrame")

	focusButton = CreateFrame("Button", "CalendarExtrasExportScrollFrameFocusButton", exportScrollFrame)
	focusButton:SetAllPoints(exportScrollFrame)
	focusButton:SetScript("OnClick", function(self, ...)
		exportEditBox:SetFocus()
	end)

	exportEditBox = CreateFrame("EditBox", "CalendarExtrasExportScrollFrameEditBox", exportScrollFrame)
	exportEditBox:SetFont(ChatFontSmall:GetFont())

	exportEditBox:SetMultiLine(true)

	exportEditBox:SetPoint("TOPLEFT")
	exportEditBox:SetWidth(263)

	exportEditBox:SetScript("OnEscapePressed", function()
		exportFrame:Hide()
	end)
	exportEditBox:SetScript("OnEditFocusGained", EditBox_HighlightText)
	exportEditBox:SetScript("OnTabPressed", TabHandler)


	exportScrollFrame:SetScrollChild(exportEditBox)
	exportScrollFrame.content = exportEditBox


	ScrollFrame_OnLoad(exportScrollFrame)

	local scrollBar = _G["CalendarExtrasExportScrollFrameScrollBar"]
	scrollBar:ClearAllPoints()
	scrollBar:SetPoint("TOPLEFT", exportScrollFrame, "TOPRIGHT", 0, -10)
	scrollBar:SetPoint("BOTTOMLEFT", exportScrollFrame, "BOTTOMRIGHT", 0, 10)
	_G["CalendarExtrasExportScrollFrameScrollBarScrollDownButton"]:SetPoint("TOP", scrollBar, "BOTTOM", 0, 4)
	_G["CalendarExtrasExportScrollFrameScrollBarScrollUpButton"]:SetPoint("BOTTOM", scrollBar, "TOP", 0, -4)
	scrollBar:Hide()

	exportScrollFrame.scrollBar = scrollBar
	exportScrollFrame.scrollBarHideable = 1

	scrollBar.Show = function(self)
		exportScrollFrame:SetPoint("BOTTOMRIGHT", exportScrollFrame:GetParent(), "BOTTOMRIGHT", -4 - self:GetWidth(), 4)
		exportScrollFrame:GetScrollChild():SetWidth(exportScrollFrame:GetWidth())
		exportScrollFrame.content:SetWidth(exportScrollFrame.defaultContentWidth)
		getmetatable(self).__index.Show(self)
	end
	scrollBar.Hide = function(self)
		exportScrollFrame:SetPoint("BOTTOMRIGHT", exportScrollFrame:GetParent(), "BOTTOMRIGHT", -4, 4)
		exportScrollFrame:GetScrollChild():SetWidth(exportScrollFrame:GetWidth())
		exportScrollFrame.content:SetWidth(exportScrollFrame.defaultContentWidth + self:GetWidth())
		getmetatable(self).__index.Hide(self)
	end

	exportScrollFrame.defaultContentWidth = exportScrollFrame.content:GetWidth()

	-- Format editbox

	exportFormatEditBox = CreateFrame("EditBox", "CalendarExtrasExportFormatEditBox", exportFrame, "InputBoxTemplate")
	exportFormatEditBox:SetAutoFocus(false)
	exportFormatEditBox:SetWidth(287)
	exportFormatEditBox:SetHeight(14)
	exportFormatEditBox:SetPoint("BOTTOMLEFT", 16, 16)

	exportFormatEditBox:SetScript("OnEnter", function(self)
		-- Todo: Localize these
		GameTooltip:ClearLines()
		GameTooltip:SetOwner(self, "ANCHOR_TOPLEFT")
		GameTooltip:AddLine("Export format:", 1, 0.82, 0)
		GameTooltip:AddLine("Set the format for each entry in the list, options are:", 1, 1, 1)
		GameTooltip:AddLine(" ")
		GameTooltip:AddDoubleLine("NAME", "Name", 1, 0.82, 0, 1, 1, 1)
		GameTooltip:AddDoubleLine("LEVEL", "Level", 1, 0.82, 0, 1, 1, 1)
		GameTooltip:AddDoubleLine("CLASS", "Class", 1, 0.82, 0, 1, 1, 1)
		GameTooltip:AddDoubleLine("STATUS", "Invite status", 1, 0.82, 0, 1, 1, 1)
		GameTooltip:AddDoubleLine("RANK", "Guild rank", 1, 0.82, 0, 1, 1, 1)
		GameTooltip:AddDoubleLine("MOD", "Moderator/leader status", 1, 0.82, 0, 1, 1, 1)
		for token, t in pairs(extraFormatTokens) do
			GameTooltip:AddDoubleLine(token, t[1], 1, 0.82, 0, 1, 1, 1)
		end
		GameTooltip:AddLine(" ")
		GameTooltip:AddLine("Use |cFFFFD200%t|r for tab. Clear the editbox to restore default string.")
		GameTooltip:AddLine("All special chars look garbled due to the font,")
		GameTooltip:AddLine("the pasted string will look correct.")
		GameTooltip:Show()
	end)
	exportFormatEditBox:SetScript("OnLeave", GameTooltip_Hide)
	exportFormatEditBox:SetScript("OnTabPressed", TabHandler)
	exportFormatEditBox:SetScript("OnEnterPressed", function(self)
		local refresh
		local text = self:GetText()
		if text ~= core.db.formatStr then
			refresh = true
		end
		if text == "" then
			core.db.formatStr = defaultFormatStr
		else
			core.db.formatStr = text
		end
		self:ClearFocus()
		if refresh then
			Export()
		end
	end)
	exportFormatEditBox:SetScript("OnEscapePressed", exportFormatEditBox.ClearFocus)
	exportFormatEditBox:SetScript("OnEditFocusLost", function(self)
		self:SetText(core.db.formatStr)
	end)
	exportFormatEditBox:SetScript("OnTextChanged", function(self)
		Export(self:GetText())
	end)
end

 --[[ ---------------------- ]]--

 do
	-- From Blizzard_Calendar.lua. Thanks for making all of them local. I have no idea if these things differ between different locales, but there's not much I can do about it right now if they do
	local CALENDAR_FULLDATE_MONTH_NAMES = {FULLDATE_MONTH_JANUARY,FULLDATE_MONTH_FEBRUARY,FULLDATE_MONTH_MARCH,FULLDATE_MONTH_APRIL,FULLDATE_MONTH_MAY,FULLDATE_MONTH_JUNE,FULLDATE_MONTH_JULY,FULLDATE_MONTH_AUGUST,FULLDATE_MONTH_SEPTEMBER,FULLDATE_MONTH_OCTOBER,FULLDATE_MONTH_NOVEMBER,FULLDATE_MONTH_DECEMBER,}
	local CALENDAR_WEEKDAY_NAMES = {WEEKDAY_SUNDAY, WEEKDAY_MONDAY, WEEKDAY_TUESDAY, WEEKDAY_WEDNESDAY, WEEKDAY_THURSDAY, WEEKDAY_FRIDAY, WEEKDAY_SATURDAY}
	local function safeselect(index, ...)
		local count = select("#", ...)
		if count > 0 and index <= count then
			return select(index, ...)
		else
			return "Unknown"
		end
	end
	local function _CalendarFrame_GetFullDate(weekday, month, day, year)
		return CALENDAR_WEEKDAY_NAMES[weekday], CALENDAR_FULLDATE_MONTH_NAMES[month], day, year, month
	end

	local rankMeta = {__index = function(self, key)
		return rawget(self, key) or ""
	end}
	function Export(format)
		local t, i = {}, 1
		local rankTable = setmetatable({}, rankMeta)

		for i = 1, GetNumGuildMembers(true) do -- This is sort of silly thing to do on each Export call (meaning each time format string is changed and so on) but what the heck...
			local name, rank = GetGuildRosterInfo(i)
			rankTable[name] = rank
		end

		while true do
			local name, level, className, classEnglish, inviteStatus, modStatus = CalendarEventGetInvite(i)
			if not name then break end
			local str = string.gsub(format or core.db.formatStr, "NAME", name)
			str = string.gsub(str, "LEVEL", level)
			str = string.gsub(str, "CLASS", className)
			str = string.gsub(str, "STATUS", inviteStatusTable[inviteStatus])
			str = string.gsub(str, "RANK", rankTable[name])
			str = string.gsub(str, "MOD", modStatus ~= "" and modStatus:sub(1, 1)..strlower(modStatus:sub(2)) or "")
			for token, t in pairs(extraFormatTokens) do
				str = string.gsub(str, token, t[2](name, classEnglish))
			end
			str = string.gsub(str, "%%t", "\t")
			t[i] = str
			i = i + 1
		end

		table.sort(t)

		local title, _, creator, eventType, _, _, textureIndex, weekday, month, day, year, hour, minute = CalendarGetEventInfo()
		tinsert(t, 1, "\""..title.."\"")
		do -- The raid name strings come from the texture cache apparently, which is all local to Blizzard_Calendar.lua, and CreateEventFrame doesn't have the text, so raid name will only be available if exporting from ViewEventFrame for now
			if CalendarViewEventFrame:IsShown() then
				tinsert(t, 2, CalendarViewEventTypeName:GetText())
			else
				tinsert(t, 2, (safeselect(eventType, CalendarEventGetTypes())))
			end
		end
		tinsert(t, 3, CALENDAR_EVENT_CREATORNAME:format(creator or UNKNOWN)) -- Won't catch creator being "", is that ever going to happen?
		tinsert(t, 4, FULLDATE:format(_CalendarFrame_GetFullDate(weekday, month, day, year)))
		tinsert(t, 5, "Starts at "..GameTime_GetFormattedTime(hour, minute, true))
		tinsert(t, 6, "")

		exportEditBox:SetText(string.join("\n", unpack(t)))
		exportEditBox:HighlightText()
	end
end

function module:AddFormatToken(token, help, func)
	extraFormatTokens[token] = {help, func}
end

function module:OnInitialize()
	SetupFrames()

	if not core.db.formatStr then core.db.formatStr = defaultFormatStr end
	exportFormatEditBox:SetText(core.db.formatStr)

	local function OnShow(self)
		exportFrame:SetHeight(self:GetHeight())
	end
	self:HookScript(CalendarViewEventFrame, "OnShow", OnShow)
	self:HookScript(CalendarCreateEventFrame, "OnShow", OnShow)

	self:RegisterEvent("CALENDAR_OPEN_EVENT", function()
		if exportFrame:IsShown() then exportFrame:Hide() end
	end)

	core:NewTab("Export", "Export the roster of this event\ninto a copy & paste-able editbox.\n\n|cFF999999Dungeon-location data incomplete if\nexported through the event-creation panel,\ndue to technical reasons.", exportFrame, Export)
end