local core = CalendarExtras

local module = core:NewModule("Invite")

 --[[ ------- Locals -------- ]]--

local title, inviteGroupButton
local text = "Invite current %s to the event"

local inviteRaidContextMenu

local InviteTimerFrame = CreateFrame("frame") -- Bleh, I should maybe piggyback another frame, or use AceTimer or something, but can't be arsed to do it now

 --[[ ------ Inviting ------ ]]--

local function GetAttendees()
	local i, t = 1, {}
	while true do
		local name = CalendarEventGetInvite(i)
		if not name then break end
		t[name] = true
		i = i + 1
	end
	return t
end

local function InGroup()
	if GetNumRaidMembers() > 0 then
		return nil, "raid"
	elseif GetNumPartyMembers() > 0 then
		return nil, "party"
	else
		return true, "raid"
	end
end

local function InviteGroupies()
	if CalendarCanSendInvite() then
		local attendees = GetAttendees()
		InviteTimerFrame.who = {}
		if GetNumRaidMembers() > 0 then
			for i = 1, GetNumRaidMembers() do
				local name = GetRaidRosterInfo(i)
				if not attendees[name] and not name:find("-") then
					tinsert(InviteTimerFrame.who, name)
				end
			end
		elseif GetNumPartyMembers() > 0 then
			for i = 1, GetNumPartyMembers() do
				local name = UnitName("party"..i)
				if not attendees[name] and not name:find("-") then
					tinsert(InviteTimerFrame.who, name)
				end
			end
		end
		if #InviteTimerFrame.who > 0 then
			InviteTimerFrame:Show()
		end
	end
end

-- What a mess... ah well
InviteTimerFrame:Hide()
InviteTimerFrame:SetScript("OnUpdate", function(self)
	if CalendarCanSendInvite() then
		local k, v = next(self.who, self.lastIndex)
		if v then
			CalendarEventInvite(v)
		else
			self:Hide()
		end
		self.lastIndex = k
	end
end)

 --[[ ---- Context Menu ---- ]]--

local function SetupContextMenu()
	inviteRaidContextMenu = CreateFrame("frame", "CalendarinviteRaidContextMenu", CalendarCreateEventInviteButton, "UIDropDownMenuTemplate")
	inviteRaidContextMenu:Hide()
	local menu = {
		[1] = {
			func = InviteGroupies,
			notCheckable = true, -- Seems it shows up checked sometimes, how odd
		},
		[2] = {
			text = "Close",
			func = function() inviteRaidContextMenu:Hide() end,
		},
	}
	function module:CalendarCreateEventInviteButtonOnClick(frame, button, ...)
		if button == "LeftButton" then
			self.hooks[frame].OnClick(frame, button, ...)
		else
			local disable, group = InGroup()
			menu[1].text = text:format(group)
			menu[1].disabled = disable
			EasyMenu(menu, inviteRaidContextMenu, "cursor", nil, nil, "MENU")
		end
	end
end

 --[[ ---------------------- ]]--

-- Update the button and title text in case group composition changes while the box is open
local function OnShowHook(self)
	local disable, group = InGroup()
	if disable then inviteGroupButton:Disable() else inviteGroupButton:Enable() end
	title:SetText(text:format(group)..":")

	module:RegisterEvent("PARTY_MEMBERS_CHANGED", OnShowHook)
	module:RegisterEvent("RAID_ROSTER_UPDATE", OnShowHook)
end

local function OnHideHook(self)
	module:UnregisterEvent("PARTY_MEMBERS_CHANGED")
	module:UnregisterEvent("RAID_ROSTER_UPDATE")
end

function module:OnInitialize()
	SetupContextMenu()

	CalendarMassInviteFrame:SetHeight(300)

	local divider = CalendarMassInviteFrame:CreateTexture("CalendarExtrasExtraDivider", "BORDER")
	divider:SetTexture([[Interface\DialogFrame\UI-DialogBox-Divider]])
	divider:SetWidth(303)
	divider:SetHeight(16)
	divider:SetPoint("BOTTOMLEFT", 10, 65)
	divider:SetTexCoord(0, 0.75390625, 0, 0.5)

	title = CalendarMassInviteFrame:CreateFontString("CalendarExtrasInviteCurrentRaidTitle", "BACKGROUND", "GameFontHighlightSmall")
	title:SetWidth(280)
	title:SetPoint("TOP", divider, "BOTTOM", 0, -4)
	title:SetJustifyH("CENTER")

	inviteGroupButton = CreateFrame("button", "CalendarExtrasInviteCurrentRaidButton", CalendarMassInviteFrame, "CalendarEventButtonTemplate")
	inviteGroupButton:SetText("Invite")
	inviteGroupButton:SetPoint("BOTTOM", 0, 19)
	inviteGroupButton:SetScript("OnClick", InviteGroupies)

	self:HookScript(CalendarMassInviteFrame, "OnShow", OnShowHook)
	self:HookScript(CalendarMassInviteFrame, "OnHide", OnHideHook)

	-- Hook to open our context menu when right-clicking the Invite button
	self:RawHookScript(CalendarCreateEventInviteButton, "OnClick", "CalendarCreateEventInviteButtonOnClick", true)
	CalendarCreateEventInviteButton:RegisterForClicks("LeftButtonUp", "RightButtonUp")

	-- Stop inviting if we change events
	core.RegisterCallback(self, "OpenEvent", function() InviteTimerFrame:Hide() end)
end